/**
 *
 *  Floor.java
 *  
 *       @author	Russell Lowke
 *      @version	Last modified 10_15_03
 *
 *    CSCIE-160:	Assignment Two
 *
 *        Usage:	java Elevator
 *
 *
 *  Description:	Floor simulation for a multi-story building.
 *
 *       Method:	unloadAndLoadPassengers() is called when elevator stops on this floor.
 *
 *        Notes:	Tab Width = 4
 *
 *					Variable prefix conventions:
 *						p 	stands for property of object, also known as a member variable or field
 *						m 	stands for "my" and means a local variable to a method
 *						Obj stands for an object from this package
 */


package cscie160.hw2;

public class Floor {

	// Static vars

    // Private vars
	private int pFloorNumber;										// This floors number.
	private int pPeopleWaiting;										// # if people waiting for elevator
	private int pPeopleOnThisFloor;									// # people who got off elevator here.

	        
	public Floor(int mFloorNumber) {
		pFloorNumber	   = mFloorNumber;
		pPeopleWaiting 	   = 0;										// construct an empty floor
		pPeopleOnThisFloor = 0;
	}
	
	
   /**
    * 
    *	Passenger on this floor is waiting for elevator [objElevator].
	*	Called by registerRequest() in class Elevator.
	*
	*	@param	objElevator		Elevator being called to this floor.
    */
	public void addPersonWaiting() {
		pPeopleWaiting++;
	}
	
	
   /**
    * 
    *	Unload elevator passengers destined for this Floor,
	*	then load waiting passengers.
    *
	*	@param	objElevator	Elevator object stopping at this floor.
    */
    public void unloadAndLoadPassengers(Elevator objElevator) {
	
		// pull off passengers from elevator
        while (objElevator.getNumPassengersDestForFloor(pFloorNumber) != 0) {	// Are there passengers to alight?
            objElevator.unloadPassenger();										// unload passenger from elevator
			pPeopleOnThisFloor++;												// put person on floor
        }

		boolean mElevatorFull = false;
		
		// push waiting passengers onto elevator
		while(pPeopleWaiting > 0 && ! mElevatorFull) {
		
			try {
				objElevator.boardPassenger(1);									// board person destined for 1st floor
				pPeopleWaiting--;												// take person off floor
			}
			
			catch(ElevatorFullException e) {
				System.out.println(e.getMessage());
				mElevatorFull = true;											// flag that elevator is full--move on
			}
		}
																	
		// set floor requests to # of passengers still waiting, this will be 0, unless the elevator is full.
		objElevator.setFloorRequests(pFloorNumber, pPeopleWaiting);
    }
}

