/**
*
*  Account.java
*  
*       @author	Russell Lowke
*      @version	Last modified 11_19_03
*
*    CSCIE-160:	Assignment Four
*
*
*  Description:	Account holds the bare minimum needed to hold state information
*				that models a bank account. A wrapper around a float type,
*				which represents the balance in the account.
*
*       Method:	Uses accessors and mutators to modify the field pBalance.
*
*		 Notes: Tab Width = 4
*
*				Variable naming prefix conventions:
*				  p   = "property" of object, indicates member variable/field
*				  m   = "my", indicates a local variable to a method
*				  obj = indicates an object from this package
*/
 

package cscie160.hw4;

public class Account {
    
    float pBalance = 0;
    
    /**																		   |
    * 
    *	Deposits an $[amount] into this account
	*	The amount may not be negative, if so an exception is thrown.
    *
	*	@param	float	Amount to be deposited into this account.
    */
	public void	deposit(float amount) throws ATMException {
    
        if (amount < 0) {
            throw new ATMException("Deposit amount is negative [$"+amount+"].");
        } else {
            pBalance += amount;
        }
    }
    
	
    /**																		   |
    * 
    *	Withdraw $[amount] from this account.
	*	The amount may not exceed the balance in the account, 
	*	if it does, then an exception is thrown.
    *
	*	@param	float	Amount to be withdrawn from this account.
    */
    public void withdraw(float amount) throws ATMException {
    
        if (amount > pBalance) {
            throw new ATMException("Withdraw amount [$"+amount+"] exceeds balance [$"+pBalance+"].");
        } else {
            pBalance -= amount;
        }
    }
    
	
    /**																		   |
    * 
    *	Accessor.  Returns the balance in the account.
    *
	*	@return	float	The balance in the account.
    */
    public float getBalance() throws ATMException {
        return pBalance;
    }
}