/**
*
*  DepositCmd.java
*  
*       @author	Russell Lowke
*      @version	Last modified 11_19_03
*
*    CSCIE-160:	Assignment Four
*
*
*  Description:	Concrete Cmd Class, used as a "go between"
*				so the Server can send a deposit() type command to 
*				ATMImlementation and adhere to the ATM Interface
*
*       Method:	Use constructor to return Cmd object
*				All Cmd objects have a doIt() call which may
*				is generically invoked
*
*		 Notes: Tab Width = 4
*
*				Variable naming prefix conventions:
*				  p   = "property" of object, indicates member variable/field
*				  m   = "my", indicates a local variable to a method
*				  obj = indicates an object from this package
*/
 

package cscie160.hw4;

import java.util.*;

class DepositCmd extends Cmd {

    /**																		   |
    * 
    *	Constructor.
	*	Establishes the ATM instance this command will be sent to
	*	ArrayList should contain 1 arg, the amount to deposit
    *
	*	@param	ATM	 	   ATM command is to be sent to.
	*	@param	ArrayList  String arg, amount to deposit.
    */
	public DepositCmd(ATM mAtm, ArrayList args) {
        super(mAtm, args); 
    }


    /**																		   |
    * 
    *	Send a deposit command to the ATM
	*	All args are stored as stings in the ArrayList, so the
	*	deposit amount must first be converted from a String to a float
	*	the Cmd abstract class demands that doIt() always return a String,
	*	in this case the string will not be used.
    *
	*	@return	String	doIt must always return a String. 
    */
	public String doIt() throws ATMException {
    
        Iterator i = pArgs.iterator();
        float depositAmount = stringToFloat( (String) i.next() );
        
        pAtm.deposit(depositAmount);
        
        return ("Deposit $" + depositAmount);
    }
}