/**
*
*  ATMImpl.java
*  
*       @author	Russell Lowke
*      @version	Last modified 12_04_03
*
*    CSCIE-160:	Assignment Five
*
*
*  Description:	This is the actual ATM that has a reference to instances
*				of class Account and manipulates their balance.
*				An instance of ATMImpl will run on the server.
*
*       Method:	Accessors and mutators that redirect call the Account.
*
*		 Notes: Tab Width = 4
*
*				Variable naming prefix conventions:
*				  f   = "field" of object, indicates member variable/field
*				  m   = "my", indicates a local variable to a method
*				  obj = indicates an object from this package
*/
 

package cscie160.hw5;

import java.rmi.*;
import java.rmi.server.UnicastRemoteObject;
import java.util.*;

public class ATMImpl extends UnicastRemoteObject implements ATM {

    ArrayList fAccountList = new ArrayList();

    /**																		   |
    * 
    *	Constructor.
	*	This ATM has three accounts
    *
    */
    public ATMImpl() throws RemoteException {
    
        super();			// invokes no-arg constructor of UnicastRemoteObject
        
        // the ATMImpl has three Accounts.
        fAccountList.add( new Account(0000001, 0));
        fAccountList.add( new Account(0000002, 100));
        fAccountList.add( new Account(0000003, 500));
    }
    
	
    /**																		   |
    * 
    *	Tells account to deposit [amount].
    *
	*	@param	float	Amount to be deposited.
    */
	public void	deposit(int mAccountNum, float mAmount) throws ATMException {
        getAccount(mAccountNum).deposit(mAmount);
    }
    
	
    /**																		   |
    * 
    *	Tells account to withdraw [amount]
    *
	*	@param	float	Amount to be withdrawn.
    */
    public void withdraw(int mAccountNum, float mAmount) throws ATMException {
        getAccount(mAccountNum).withdraw(mAmount);
    }
    
	
    /**																		   |
    * 
    *	Asks account for the balance in that account.
    *
	*	@return	float	Balance in [account]
    */
    public float getBalance(int mAccountNum) throws ATMException {
        return getAccount(mAccountNum).getBalance();
    }
    
    
	/**																		   |
    * 
    *	Asks account for the balance in that account.
    *
	*	@return	float	Balance in [account]
    */
    public Account getAccount(int mAccountNum) throws ATMException  {
    
        Account mReturn  = null;
        Account mAccount = null;
        
        Iterator mItr = fAccountList.iterator();
        while(mItr.hasNext() && mReturn == null) {
            mAccount = (Account) mItr.next();
            if (mAccount.getAccountNum() == mAccountNum)
                mReturn = mAccount;
		}
        
        if (mReturn == null)
            throw new ATMException("ATM cannot find an account #" + mAccountNum);
            
        return mReturn;
    }
}