/**
*
*  TSWithdrawer.java
*  
*       @author	Russell Lowke
*      @version	Last modified 12_28_03
*
*    CSCIE-160:	Assignment Six
*
*
*  Description:	This is the actual ATM that has a reference to an instance
*				of class Account and manipulates its balance.
*				An instance of AccountHolder will run on the server.
*
*       Method:	Accessors and mutators that redirect call the Account.
*
*		 Notes: Tab Width = 4
*
*				Variable naming prefix conventions:
*				  f   = "field" of object, indicates member variable/field
*				  m   = "my", indicates a local variable to a method
*				  obj = indicates an object from this package
*/
 
// Thread-safe version 

package cscie160.hw6;


public class TSWithdrawer extends AccountHolder {
    
    /**																		   |
    * 
    *	Constructor.
    *
    */
	public TSWithdrawer(Account mAccount) {
        super(mAccount);
    }

    public void run() {
        // 
        // non-thread-safe implementation that iterates a fixed number of times
        // making a fixed withdrawal on fAccount.  That fixed amount should be 
        // the same amount that Depositor deposits.  amount in the account should
        // be the same as it was prior to all operations and oney is preserved.
        
        try {
            for (int i = 0; i < ITERATIONS; i++) {
            
                // check the balance on fAccount to verify that there are sufficient
                // funds to make the withdrawal, if not, Withdrawer should sleep for a short
                // interval and recheck the balance [loop, contingent on balance becoming sufficient.]
                
                // TS before checking balance on account it first obtains the monitor on that Account
                synchronized (this) {
                    int attempts = 0;  // number of check attempts made before withdrawing
                    
                    while(fAccount.getBalance() < INITIAL_DEPOSIT && attempts < CHECKTESTS) {
                        attempts++;
                        try {
                            wait(SLEEPTIME);
                        } catch (InterruptedException e) {
                            Thread.currentThread().interrupt();
                        }
                    }
                    
                    fAccount.withdraw(INITIAL_DEPOSIT);
                    notifyAll();  // Is this needed?
                }
            }
        } catch (ATMException e) {
            System.out.println(e.getMessage());
        }
    }
}