/**																			   |
*
*  BankImpl.java
*  
*       @author	Russell Lowke
*      @version	Last modified 01_14_04
*
*    CSCIE-160:	Final Project
*
*
*  Description:	The Bank is a remote object that hosts remote Account objects. 
*				It should provide a method to obtain a reference to a remote
*				Account.
*
*       Method:	Uses accessors and mutators to modify the field fBalance.
*
*		 Notes: Tab Width = 4
*
*				Variable naming prefix conventions:
*				  f   = "field" of object, indicates member variable/field
*				  m   = "my", indicates a local variable to a method
*				  obj = indicates an object from this package
*/

package cscie160.project;

import java.rmi.*;
import java.rmi.server.UnicastRemoteObject;
import java.util.*;


public class BankImpl extends UnicastRemoteObject implements Bank {
    
    ArrayList fAccountList = new ArrayList();
    
    /**																		   |
    * 
    *	Constructor.
    *
    */
    public BankImpl() throws RemoteException {
    
        /*
        // 	the Bank has three Accounts.
        You may assume the following authentication values.
        Account PIN
        0000001	1234
        0000002	2345
        0000003	3456

        And you may assume the following authorization values.
        Account Number	Deposit	Withdraw	Balance
        0000001	Permitted	Permitted	Permitted
        0000002	Permitted	Not Permitted	Permitted
        0000003	Not Permitted	Permitted	Permitted
        */
        
        fAccountList.add(new AccountImpl(0000001, 1234,   0, true,  true,  true));
        fAccountList.add(new AccountImpl(0000002, 2345, 100, true,  false, true));
        fAccountList.add(new AccountImpl(0000003, 3456, 500, false, true,  true));
    }
    
    
	/**																		   |
    * 
    *	Retrieve a specific account.
    *
	*	@return	Account		the account reference requested
    */
    public Account getAccount(int mAccountNum) throws SecurityException, RemoteException  {
    
        AccountImpl mReturn  = null;
        AccountImpl mAccount = null;
        
        Iterator mItr = fAccountList.iterator();
        while(mItr.hasNext() && mReturn == null) {
            mAccount = (AccountImpl) mItr.next();
            if ( mAccount.getAccountNum() == mAccountNum)
                mReturn = mAccount;
		}
        
        if (mReturn == null)
            throw new SecurityException("Bank cannot find an account #" + mAccountNum);
        
        return (Account) mReturn;
    }
}