/**																			   |
*
*  Client.java
*  
*       @author	Russell Lowke
*      @version	Last modified 01_14_04
*
*    CSCIE-160:	Final Project
*
*        Usage:	java Client
*
*
*  Description:	This class is largely supplied, here are Charlie's instructions:
*
*				"The Client should have a main that obtains a connection to an 
*				 ATM using the ATMFactory. Then the Client should register itself
*				 with the ATM as an ATMListener. Finally, it should call the 
*				 testATM method defined below. This method calls the other test 
*				 methods below to simulate various scenarios. 
*
*				 You will also need to declare a helper method getAccountInfo 
*				 that creates an AccountInfo object given the appropriate account
*				 information.
*
*				 The Client should also implement the ATMListener interface and 
*				 its notification handler method should print out any 
*				 TransactionNotifications it receives."
*
*		 Notes: Tab Width = 4
*
*				Variable naming prefix conventions:
*				  f   = "field" of object, indicates member variable/field
*				  m   = "my", indicates a local variable to a method
*				  obj = indicates an object from this package
*/


package cscie160.project;

import java.net.MalformedURLException;
import java.rmi.Naming;
import java.rmi.NotBoundException;
import java.rmi.RemoteException;
import java.rmi.UnknownHostException;

import java.util.EventObject;


public class Client extends java.rmi.server.UnicastRemoteObject implements ATMListener {

	public Client() throws java.rmi.RemoteException {
        super();
    }
    
	public static void testATM(ATM atm) {
        if (atm!=null) {
            printBalances(atm);
            performTestOne(atm);
            performTestTwo(atm);
            performTestThree(atm);
            performTestFour(atm);
            performTestFive(atm);
            performTestSix(atm);
            performTestSeven(atm);
            performTestEight(atm);
            performTestNine(atm);
            printBalances(atm);
        }
	}
    
	public static void printBalances(ATM atm) {        
        try {
            System.out.println("Balance(0000001): "+atm.getBalance(getAccountInfo(0000001, 1234)));
            System.out.println("Balance(0000002): "+atm.getBalance(getAccountInfo(0000002, 2345)));
            System.out.println("Balance(0000003): "+atm.getBalance(getAccountInfo(0000003, 3456)));
        } catch (Exception e) {
            e.getMessage();
            e.printStackTrace();
        }
	}
   
	public static void performTestOne(ATM atm) {
         
        try {
            atm.getBalance(getAccountInfo(0000001, 5555));
        } catch (Exception e) {
            System.out.println("Failed as expected: "+e);
        }
	}
	public static void performTestTwo(ATM atm) {
        
        try {
            atm.withdraw(getAccountInfo(0000002, 2345), 500);
        } catch (Exception e) {
            System.out.println("Failed as expected: "+e);
        }
	}
	public static void performTestThree(ATM atm) {
        
        try {
            atm.withdraw(getAccountInfo(0000001, 1234), 50);
        } catch (Exception e) {
            System.out.println("Failed as expected: "+e);
        }
	}
	public static void performTestFour(ATM atm) {
        
        try {
            atm.deposit(getAccountInfo(0000001, 1234), 500);
        } catch (Exception e) {
            System.out.println("Unexpected error: "+e);
        }
	}
	public static void performTestFive(ATM atm) {
        
        try {
            atm.deposit(getAccountInfo(0000002, 2345), 100);
        } catch (Exception e) {
            System.out.println("Unexpected error: "+e);
        }
	}
	public static void performTestSix(ATM atm) {
        
        try {
            atm.withdraw(getAccountInfo(0000001, 1234), 100);
        } catch (Exception e) {
            System.out.println("Unexpected error: "+e);
        }
	}
	public static void performTestSeven(ATM atm) {
        
        try {
            atm.withdraw(getAccountInfo(0000003, 3456), 300);
        } catch (Exception e) {
            System.out.println("Unexpected error: "+e);
        }
	}
	public static void performTestEight(ATM atm) {
                    
        try {
            atm.withdraw(getAccountInfo(0000001, 1234), 200);
        } catch (Exception e) {
            System.out.println("Failed as expected: "+e);
        }
	}
	public static void performTestNine(ATM atm) {
        
        try {
            atm.transfer(getAccountInfo(0000001, 1234), getAccountInfo(0000002, 2345), 100);
        } catch (Exception e) {
            System.out.println("Unexpected error: "+e);
        }
	}

    
    

	/**																		   |
    * 
    *	as suggested, helper method getAccountInfo 
    *	that creates an AccountInfo object given the appropriate account
    *	information.
    *
    *	@param	int		Account number requested
	*	@param	int		Account pin requested
    *
    *	@return AccountInfo wrapper object returned
    */
    public static AccountInfo getAccountInfo(int mAccountNum, int mAccountPin) {
    
        // create an accountInfo object
        return new AccountInfo(mAccountNum, mAccountPin);
    }
    
    
	/**																		   |
    * 
    *	as suggested, notification handler method of ATMListener prints out any 
	*	TransactionNotifications it receives.
    *
    *	@param	EventObject		the distributed TransactionNotification
    */
	public void handleEvent(EventObject mTransaction) throws java.rmi.RemoteException {
    
        // print the transaction
        System.out.println("<< " + mTransaction + " >>");
    }
    
    
	/**																		   |
    * 
    *	main()
    *
    */
    public static void main(String[] args) {
    
        ATM mAtm = null;
        
        try {
            ATMFactory mFactory = (ATMFactory) Naming.lookup("//localhost/atmfactory");
            mAtm = mFactory.getATM();
        } catch (MalformedURLException mue) {
            mue.printStackTrace();
        } catch (NotBoundException nbe) {
            nbe.printStackTrace();
        } catch (UnknownHostException uhe) {
            uhe.printStackTrace();
        } catch (RemoteException re) {
            re.printStackTrace();
        }
        
        if (mAtm != null) {
            try {
            
                // register self with the ATM as an ATMListener
                Client mClient = new Client();
                mAtm.addListener(mClient);
            
                testATM(mAtm);
                
            } catch (RemoteException re) {
                System.out.println("An exception occurred while communicating with the ATM");
                re.printStackTrace();
            } catch (Exception e) {
                System.out.println(e.getMessage());
            }
        }
    }
}