// REMOTE OBJECT

package examples.hello;

import java.rmi.*;
import java.rmi.server.UnicastRemoteObject;


// 1. Implement a Remote Interface
// (option) can indicate the remote server it is extending
// public class HelloImpl extends java.rmi.server.UnicastRemoteObject implements Hello {

// If you choose to extend a remote object from a nonremote class, you need to explicitly
// export the remote object by calling the method UnitcastRemoteObject.exportObject
public class HelloImpl extends UnicastRemoteObject implements Hello {

    private String name;
    
    // 2. Define Cinstructor.
    // must throw RemoteException as, if communication resource are not available
    // it might fail
    public HelloImpl(String s) throws RemoteException {
        super();			// invokes no-arg constructor of UnicastRemoteObject
                            // which "exports" it by listening for incoming calls
        name = s;
    }
    
    // 3. Provide Implementation for each Remote Method
    // Arguments to, or return values from remote methods can be of any type,
    // as long as those objects implement the interface java.io.Serializable
    public String sayHello() throws RemoteException {
        return "Hello World!";
    }
    
    public static void main(String args[]) {
    
        // 4. Create and install a security manager
        // System.setSecurityManager(new RMISecurityManager());
        // if used will need a "policy file" in which you can specify your security policy.
        //   grant {
        //       // Allow everything for now
        //       permission java.security.AllPermission;
        //   };
        // You tell the JVM where to find this file by a command line argument when invoking Java: 
        // java -Djava.security.policy=filename foo.bar.MyClass


        try {
            
            // 5. Create one or more instances of remote object
            HelloImpl obj = new HelloImpl("HelloServer");
            // the constructor exports the remote object, which means that 
            // once created, the remote object is ready to begin listening
            // for incoming calls
            
            // 6. Register remote object
            Naming.rebind("//localhost/HelloServer", obj);
            System.out.println("HelloServer bound in registry");
            // for caller to be able to invoke a method on a remote object,
            // that caller must first obtain a refernce to it.
            // (option) a port number can be supplied, for example
            //  Naming.rebind("//myhost:1234/HelloServer", obj);
            //  the port defaults to 1099.  It is necessary to specify the port
            //  only if a server creats a registry on a port other than the default 1099
            
            
            
        } catch (Exception e) {
            System.out.println("HelloImpl err:" + e.getMessage());
            e.printStackTrace();
        }
    }
}