/**
 *
 *  Elevator.java
 *  
 *       @author	Russell Lowke
 *      @version	Last modified 09_29_03
 *
 *    CSCIE-160:	Assignment One
 *
 *        Usage:	java Elevator
 *
 *
 *  Description:	Part 1 of elevator simulation for a multi-story building.
 *
 *       Method:	Use boardPassenger(floorDest), to add each elevator passenger 
 *						with a floor destination of [floorDest], then use
 *						go() to start the elevator. go() will cause the elevator
 *    					to search for the nearest requested floor and drop off passengers for
 *   					that floor, then proceed to the next nearest requested destination until
 *						all passengers have been dropped off.
 *
 *        Notes:	Tab Width = 4
 *
 */


package cscie160.hw1;

public class Elevator {

	// Static vars
	public static final int CAPACITY   = 10;						// maximum number of people in evevator
	public static final int NUM_FLOORS = 7;							// # of floors in building

    // Private vars
	private int currentFloor;										// floor elevator is currently on
    private int travelDirection;
    private int passengers;											// # of passengers in elevator
    private int [] passengerDest;									// # of passengers requesting each floor
    private int [] floorRequests;									// destination requests for each floor
	        
	public Elevator() {
		
		
		currentFloor		   = 1;									
		travelDirection		   = +1;								// 0 = stopped, 1 = up, -1 = down
		passengers			   = 0;									
		passengerDest	   = new int[NUM_FLOORS + 1];				//   Note: passengerDest[0] unused									
		floorRequests	   = new int[NUM_FLOORS + 1];				//   Note: floorRequests[0] unused
																		
	}
	
   /**
	*
    *	Move elevator one floor in the travelDirection.
    *
    */
    public void move() {
        
        currentFloor += travelDirection;
            
        // ensure floor is legal
        if (currentFloor < 1) {
            currentFloor    = 1;									// elevator at botton, so stop
            travelDirection = 0;
        }
        else if (currentFloor > NUM_FLOORS) {
            currentFloor    = NUM_FLOORS;							// elevator at top, so stop
            travelDirection = 0;
        }
    }
        
   /**
    * 
    *	Board a passenger destined for a floor.
    * 
    *	@param	destinationFloor	The floor that passenger wants to go to.
    */
    public void boardPassenger(int destinationFloor) {
    
        // ensure destination within bounds (throw exception if not??)
        if (destinationFloor >= 1 && destinationFloor <= NUM_FLOORS) {
            passengers++;
            passengerDest[destinationFloor]++;
            floorRequests[destinationFloor]++;
        }
    }
        
        
   /**
    * 
    *	Unload a passenger.
    *
    */
    public void unloadPassenger() {
        passengers--;
        passengerDest[currentFloor]--;
    }
        
        
   /**
    * 
    *	Stop the elevator and unload passengers destined for this floor.
    *
    */
    public void stop() {
    
        travelDirection = 0;										// stop
        System.out.println("Stopping on floor " + (currentFloor));
        
        // unload all passengers for this floor
        while (passengerDest[currentFloor] != 0) {					// loop through passenger destinations
        
            unloadPassenger();										// unload a passenger
        }
        
		System.out.println(this);									// print new elevator state
		System.out.println();
			
        floorRequests[currentFloor] = 0;							// clear floor requests for this floor
    }
        
        
   /**
    * 
    * 	Return the direction of closest destination floor,
    * 	with preference for moving down.
    *
    *	@return		Direction of closest destination floor.
    */
    public int findDirection() {
        
        int targetDirection = 0;									// direction elevator should move
        int modifier        = 1;									// modifier to check for destination floor
        
		int testFloorUp;											// floor above being tested for
		int testFloorDwn;											// floor below being tested for
		
        while (targetDirection == 0 && modifier < NUM_FLOORS) {		// loop until target found
            
            // floors to test 
            testFloorUp  = currentFloor + modifier;
            testFloorDwn = currentFloor - modifier;
            
            // keep tests within bounds
            if (testFloorUp > NUM_FLOORS)
                testFloorUp = NUM_FLOORS;
            if (testFloorDwn < 1)
                testFloorDwn = 1;
            
            // test for floor
            if (floorRequests[testFloorDwn] > 0)
                targetDirection = -1;								// going down
            else if (floorRequests[testFloorUp] > 0)
                targetDirection = +1;								// going up
            else
                modifier++;											// check next
        }
        
        return targetDirection;
    }
 
 
   /**
    * 
    * 	Drive elevator to next destination.
    *
    */ 
    public void go() {
        
        travelDirection = findDirection();							// get new direction of travel
        
        while(travelDirection != 0) {
            move();													// move me
            if (floorRequests[currentFloor] != 0) {					// check for floor requests on this floor
                stop();												// stop elevator and load/unload
                travelDirection = findDirection();					// get new direction of travel
            }    
        }
    }
    
        
   /**
    * 
    *	Override the toString() method of java.lang.Object for printing Elevator values.
	*
    *	@return		summary of pertinent values in elevartor.
    */
	public String toString() {
            String mReturn = "Currently " + passengers + " passengers onboard \n" + 
                             "Current Floor: " + (currentFloor);
            return mReturn;
	}
        
        
   /**
    * 
    * 	Create an elevator and board two passengers into it.
    *
    */	
	static public void main(String argv[]) {

		Elevator mElevator = new Elevator();						// make an elevator
            
		for (int i = 1; i < 30; i++) {								// drive elevator up and down 30 times
		
			mElevator.travelDirection = +1;							// manually force travel direction to up
																	// should probably use a setter method
																	
			for (int j = 1; j < NUM_FLOORS; j++) {					// drive elevator to top floor
				mElevator.move();
			}
			
			mElevator.travelDirection = -1;							// manually force travel direction to down
																	// should probably use a setter method
																	
			for (int j = 1; j < NUM_FLOORS; j++) {					// drive elevator to bottom floor
				mElevator.move();
			}
		}
		
		mElevator.boardPassenger(2);								// passenger destined for floor 2.
		mElevator.boardPassenger(3);								// passenger destined for floor 3.
		System.out.println(mElevator);
		System.out.println();
		mElevator.go();												// Do elevator thing.
    }
}

