 /**
 *
 *  Floor.java
 *  
 *       @author	Russell Lowke
 *      @version	Last modified 11_03_03
 *
 *    CSCIE-160:	Assignment Three
 *
 *        Usage:	java Elevator
 *
 *
 *
 *  Description:	Floor simulation for a multi-story building.
 *
 *       Method:	unloadAndLoadPassengers() is called when elevator stops on this floor.
 *					addPerson() is called to create a Passenger object witrh a destination floor.
 *
 *        Notes:	Tab Width = 4
 *
 *					Variable prefix conventions:
 *						p 	stands for property of object, also known as a member variable or field
 *						m 	stands for "my" and means a local variable to a method
 *						Obj stands for an object from this package
 */


package cscie160.hw3;

import java.util.*;

public class Floor {

	// Static vars

    // Private vars
	private int pFloorNumber;										// This floors number.
	
	private ArrayList pUpPsngrs;									// list of Passenger objects waiting for Up
	private ArrayList pResidents;									// list of Passenger objects resident here
	private ArrayList pDwnPsngrs;									// list of passenger objects waiting for Dwn

	        
	public Floor(int mFloorNumber) {
		pFloorNumber	   	= mFloorNumber;
		
		pUpPsngrs     		= new ArrayList();
		pResidents    		= new ArrayList();
		pDwnPsngrs    		= new ArrayList();
		
	}
	
	
    /**
    * 
    *	Create a Passenger on this floor, wanting to get to [mDestFloor].
	*	fctn called by newPerson() in class Elevator.
	*
	*	@param	mDestFloor		floor number Passenger is trying to get to.
    */
	public void addPerson(int mDestFloor) {
	
		Passenger objPerson = new Passenger(pFloorNumber, mDestFloor);
		
		addPerson(objPerson);
	}
	
	
    /**
    * 
    *	Put a Passenger instance into one of three ArrayLists,
	*	depending on which Floor the passenger is trying to get to
	*
	*	@param	objPerson		reference to Passenger object being added to this Floor.
    */
	public void addPerson(Passenger objPerson) {
	
		int mDestFloor = objPerson.getDestFloor();
		
		if (mDestFloor < pFloorNumber) {
			pDwnPsngrs.add(objPerson);
		} else if (mDestFloor == pFloorNumber) {
			pResidents.add(objPerson);
		} else if (mDestFloor > pFloorNumber) {
			pUpPsngrs.add(objPerson);
		}
	}
	
	
   /**
    * 
    *	Unload elevator passengers destined for this Floor,
	*	then load waiting passengers, into elevator.
    *
	*	@param	objElevator	Elevator object stopping at this floor.
    */
    public void unloadAndLoadPassengers(Elevator objElevator) {
	
		// unload passengers from elevator
		objElevator.unloadPassengers();
		
		// push waiting passengers onto elevator
		Passenger objPerson;
		Iterator mIter = pUpPsngrs.iterator();
		if (objElevator.getTravelDirection() == -1) {
			mIter = pDwnPsngrs.iterator();
		}
		
		while(mIter.hasNext()) {
			objPerson = (Passenger) mIter.next();
			try {
				objElevator.boardPassenger(objPerson);
				mIter.remove();
			}
			catch(ElevatorFullException e) {
				System.out.println(e.getMessage());
			}
		}
		
		// set floor requests to # of passengers still waiting.
		int mPassengersWaiting = pUpPsngrs.size() + pDwnPsngrs.size();
		objElevator.setFloorRequests(pFloorNumber, mPassengersWaiting);
		
		if (mPassengersWaiting > 0)
			System.out.println(mPassengersWaiting + " people remain waiting on this floor.");
			
		System.out.println(objElevator);					// print new elevator state
		
    }
    
	/**
    * 
    *	Return the number of passengers wanting to go up
	*	called by findDirection() in class Elevator
    *
	*	@return	int		number of passengers on this floor wanting to go up.
    */
    public int getNumUpPsngrs() {
    	return pUpPsngrs.size();
    }

	/**
    * 
    *	Return the number of passengers wanting to go down
	*	called by findDirection() in class Elevator
    *
	*	@return	int		number of passengers on this floor wanting to go down.
    */
    public int getNumDwnPsngrs() {
    	return pDwnPsngrs.size();
    }
}

