/**
*
*  Cmd.java
*  
*       @author	Russell Lowke
*      @version	Last modified 11_19_03
*
*    CSCIE-160:	Assignment Four
*
*
*  Description:	The abstract base class used for all commands sent from the Server class
*				to the ATMImplementation class. Member variables are kept for the
*				specific ATM to which the command is being sent, and the args to be
*				sent to it.
*
*       Method:	Use constructor to return Cmd object
*				All Cmd objects have a doIt() call which may
*				is generically invoked
*
*		 Notes: Tab Width = 4
*
*				Variable naming prefix conventions:
*				  p   = "property" of object, indicates member variable/field
*				  m   = "my", indicates a local variable to a method
*				  obj = indicates an object from this package
*/
 

package cscie160.hw4;

import java.util.*;

public abstract class Cmd
{
	public ATM			pAtm;
	public ArrayList	pArgs;

    /**																		   |
    * 
    *	Constructor for abstract command class.
	*	All commands require an ATM to which the command will be sent
	*	and and args list of arguments/parameters used by the command.
    *
	*	@param	ATM			The target ATM to send this command to
	*	@param	ArrayList	Args list of values.
    */
	public Cmd(ATM mAtm, ArrayList mArgs) {
        pAtm  = mAtm;
        pArgs = mArgs;
	}


    /**																		   |
    * 
    *	Empty method declaration.
	*	All commands must have a doIt() method.
    *
	*	@return	String	number of passengers on this floor wanting to go up.
    */
	public abstract String doIt() throws ATMException;
  
  
    /**																		   |
    * 
    *	Converts a String to a float.
    *
	*	@param	String	The string to convert.
	*	@return	float	A float representation of the given string.
    */
	protected static float stringToFloat(String stringObject) {
		return Float.parseFloat(stringObject.trim());
	}
}
