
/// USAGE Example

// to create skeletons and stubs
//    rmic -classpath `pwd` rmiEcho.EchoImpl

// to start registery
//    rmiregistry 8000 &

// to run on Server
//    java rmiEcho.RmiEchoServer localhost 8000 Impl &

// to run on Client
//    java rmiEcho.RmiEchoClient localhost Impl


package rmiEcho;

import java.rmi.registry.*;

public class RmiEchoServer {

	public static void main(String argv[]) {
    
        if(argv.length != 3) {
            System.out.println("Usage: java RmiEchoServer hostname portNumber objectName");
            System.exit(1);
        }
        
        try {
            EchoImpl impl = new EchoImpl();
            int port = 0;
            
            try {
                port = Integer.parseInt(argv[1]);
            } catch (NumberFormatException e) {
                System.exit(1);
            }
            
            Registry registry = LocateRegistry.getRegistry(argv[0], port);
            if(registry == null) {
                System.out.println("getRegistry returned null");
                System.exit(1);
            }
            
            System.out.println("Registering EchoImpl object on host " + argv[0] + 
                               ", port " + argv[1] + 
                               ", under name " + argv[2]);
            registry.rebind(argv[2], impl);
            System.out.println("EchoImpl object was registered under name " + argv[2]);
            
        } catch (Exception e) {
            e.printStackTrace();
        }
        
        System.out.println("main exiting");
	}
}
