/**
*
*  TSWithdrawer.java
*  
*       @author	Russell Lowke
*      @version	Last modified 12_28_03
*
*    CSCIE-160:	Assignment Six
*
*
*  Description:	Supplied class
*
*       Method:	Accessors and mutators that redirect call the Account.
*
*		 Notes: Tab Width = 4
*/
 
package cscie160.hw6;

// import java.lang.Thread;

public class WildAccounting implements ATMParams
{
    static boolean debug = false;

    private static void shakeAndBake(boolean threadSafe) throws Exception {
    
        // We create our Thread in our own ThreadGroup in order to monitor
        // when all the individual threads have run to completion
        ThreadGroup myThreadGroup = new ThreadGroup("WildAccounting");
        
        // Upper case identifiers are constants defined in ATMParams
        int withdrawerType = threadSafe ? TSWITHDRAWER : WITHDRAWER;
        int depositorType  = threadSafe ? TSDEPOSITOR  : DEPOSITOR;
        Account account = threadSafe ? new TSAccount(INITIAL_DEPOSIT) : new Account(INITIAL_DEPOSIT);
        
        float initialAmount 	   = account.getBalance();
        final int numberOfThreads  = 10;
        Thread depositorThreads[]  = new Thread[numberOfThreads];
        Thread withdrawerThreads[] = new Thread[numberOfThreads];
        
        for(int i = 0; i < numberOfThreads; i++) {
            depositorThreads[i] = new Thread(myThreadGroup, 
					     AccountHolder.createAccountHolder(depositorType, account));
            withdrawerThreads[i] = new Thread(myThreadGroup,
					      AccountHolder.createAccountHolder(withdrawerType, account));
                          
            // A good place to set priorities
            depositorThreads[i].setPriority(MMIN_PRIORITY);
            withdrawerThreads[i].setPriority(MMAX_PRIORITY);
        }
        
        for(int i = 0; i < numberOfThreads; i++) {
            depositorThreads[i].start();
            withdrawerThreads[i].start();
        }
        
        // Return only after all the Threads we created have died
        try {
            while(myThreadGroup.activeCount()> 0) {
                if(debug) {
                    System.out.println("activeCount = " + myThreadGroup.activeCount());
                    for(int i = 0; i < numberOfThreads; i++) {
                        if(depositorThreads[i].isAlive())
                            System.out.println("Depositor # " + i + " is alive");
                        if(withdrawerThreads[i].isAlive())
                            System.out.println("Withdrawer # " + i + " is alive");
                    }
                }
                Thread.sleep(5000);
            }
        } catch(Exception e) { }
        
        if(initialAmount == account.getBalance())
            System.out.println("\n==>Money was preserved\n");
        else
            throw new Exception("Money not preserved!");
            
	}
        
	public static void main(String argv[]) {
    
        try {
            System.out.println("==> Using unsafe accounting");
            shakeAndBake(false);
        } catch (Exception e) { System.out.println(e); }
            
        try {
            System.out.println("==>Using threadsafe accounting");
            shakeAndBake(true);
        } catch (Exception e) { System.out.println("==> " + e); }
    }
}