/**																			   |
*
*  AccountImpl.java
*  
*       @author	Russell Lowke
*      @version	Last modified 01_14_04
*
*    CSCIE-160:	Final Project
*
*
*  Description:	The Account is a remote object that provides methods for adding
*				and subtracting funds from the Account. 
*				It verifies that appropriate funds are available in the Account. 
*
*       Method:	Uses accessors and mutators to modify the field fBalance.
*
*		 Notes: Tab Width = 4
*
*				Variable naming prefix conventions:
*				  f   = "field" of object, indicates member variable/field
*				  m   = "my", indicates a local variable to a method
*				  obj = indicates an object from this package
*/

package cscie160.project;

import java.rmi.*;
import java.rmi.server.UnicastRemoteObject;
import java.util.*;

public class AccountImpl extends UnicastRemoteObject implements Account {
    
    int		fAccountNum		= 0000000;
    int		fAccountPin     = 0000;
    float 	fBalance		= 0;
	boolean	fAuthDeposit	= false;
    boolean	fAuthWithdraw	= false;
    boolean	fAuthBalance	= false;
    
    
    /**																		   |
    * 
    *	Constructor.
    *
    */
    public AccountImpl(int mAccountNum, 
                       int mAccountPin,
                     float mBalance,
                   boolean mAuthDeposit,
                   boolean mAuthWithdraw,
                   boolean mAuthBalance) throws RemoteException {
                   
        fAccountNum     = mAccountNum;
        fAccountPin     = mAccountPin;
        fBalance        = mBalance;
        fAuthDeposit    = mAuthDeposit;
        fAuthWithdraw   = mAuthWithdraw;
        fAuthBalance    = mAuthBalance;
    }
    
    
    /**																		   |
    * 
    *	Deposits an $[amount] into this account
	*	The amount may not be negative, if so an exception is thrown.
    *
	*	@param	float	Amount to be deposited into this account.
    */
	public void deposit(float mAmount) throws ATMException, RemoteException {
    
        if (mAmount < 0) {
            throw new ATMException("Deposit amount is negative [$"+mAmount+"].");
        } else {
            fBalance += mAmount;
        }
    }
    
	
    /**																		   |
    * 
    *	Withdraw $[amount] from this account.
	*	The amount may not exceed the balance in the account, 
	*	if it does, then an exception is thrown.
    *
	*	@param	float	Amount to be withdrawn from this account.
    */
    public void withdraw(float mAmount) throws ATMException, RemoteException {
    
        if (mAmount > fBalance) {
            throw new ATMException("Withdraw amount [$"+mAmount
                                        +"] exceeds balance [$"+fBalance+"].");
        } else {
            fBalance -= mAmount;
        }
    }
    
	
	/**																		   |
    * 
    *	Transfer $[amount] from this account into mTarget account
	*	The amount may not exceed the balance in this account, 
	*	if it does, then an exception is thrown.
    *
	*	@param	float	Amount to be withdrawn from this account.
    */
    public void transfer(float mAmount, Account mTarget)
                                        throws ATMException, RemoteException {
    
        // withdraw transfer amount from this account
        withdraw(mAmount);
        
        // .. and deposit into the target account
        mTarget.deposit(mAmount);
    }
    
    
	/**																		   |
    * 
    *	Accessors.
    *
    */
    
	public float getBalance() {
        return fBalance;
    }
    public int getAccountNum() {
        return fAccountNum;
    }
	public int getAccountPin() {
        return fAccountPin;
    }
	public boolean getAuthBalance() {
        return fAuthBalance;
    }
	public boolean getAuthDeposit() {
        return fAuthDeposit;
    }
	public boolean getAuthWithdraw() {
        return fAuthWithdraw;
    }
}