/**																			   |
*
*  BankServer.java
*  
*       @author	Russell Lowke
*      @version	Last modified 01_14_04
*
*    CSCIE-160:	Final Project
*
*        Usage:	java BankServer
*
*
*  Description:	The BankServer starts up and creates a Bank instance and a
*				Security instance. It then registers each with the naming
*				service (tnameserv or rmiregistry) to make each accessible by
*				remote lookup.
*
*		 Notes: Tab Width = 4
*
*				Variable naming prefix conventions:
*				  f   = "field" of object, indicates member variable/field
*				  m   = "my", indicates a local variable to a method
*				  obj = indicates an object from this package
*/

package cscie160.project;

import java.rmi.*;
import java.rmi.server.UnicastRemoteObject;

public class BankServer {
    
    /**																		   |
    * 
    *	main()
    *
    */
    public static void main(String[] argv) {
    
        Bank	 mBankImpl;
        Security mSecurityImpl;
        
        // Create and install a security manager  (not used)
        // System.setSecurityManager(new RMISecurityManager());


        try {
        
            // Create instance of bank object
            mBankImpl = new BankImpl();
            
            // Register remote object
            Naming.rebind("//localhost/bank", mBankImpl);
            System.out.println("bank bound in registry");
            
            
            // Create instance of security object
            mSecurityImpl = new SecurityImpl(mBankImpl);
            
            // Register remote object
            Naming.rebind("//localhost/security", mSecurityImpl);
            System.out.println("security bound in registry");
            
        } catch (Exception e) { 
            System.out.println(e.getMessage());
            e.printStackTrace();
        }
    }
}