/**																			   |
*
*  TransactionNotification.java
*  
*       @author	Russell Lowke
*      @version	Last modified 01_14_04
*
*    CSCIE-160:	Final Project
*
*
*  Description:	The TransactionNotification object is a data class that includes
*				pertinent information about a particular transaction including
*				target accounts and amounts. It has a toString() method that
*				returns a String describing the transaction that can be easily
*				printed by any listener. Instances are passed by value between
*				processes.
*
*       Method:	Uses accessors and mutators to modify the field fBalance.
*
*		 Notes: Tab Width = 4
*
*				Variable naming prefix conventions:
*				  f   = "field" of object, indicates member variable/field
*				  m   = "my", indicates a local variable to a method
*				  obj = indicates an object from this package
*/


package cscie160.project;

import java.io.*;
import java.util.*;


public class TransactionNotification extends EventObject
            implements java.io.Serializable, ATMParams {
    
	// It may seem redundant to have a reference to the originator, source,
	// since java.util.EventObject has a reference to its source, but that 
	// one is declared transient, and we intend to send these to remote calls
    protected Object fSource;
    
    int         fTransactionType = 0;
    AccountInfo fSourceAcc       = null;
	AccountInfo fTargetAcc       = null;
    float       fAmount          = 0;
    
    
    /**																		   |
    * 
    *	Constructor
    *
    */
	public TransactionNotification(Object mSrc,
                                  int mTransactionType,
                          AccountInfo mSourceAcc,
                          AccountInfo mTargetAcc,
                                float mAmount) {
                                
        super(mSrc); // see BabyEvent.java...
        fSource = mSrc;
        
        fTransactionType = mTransactionType;
        fSourceAcc       = mSourceAcc;
        fTargetAcc       = mTargetAcc;
        fAmount          = mAmount;
    }
    
    
	/**
    * 
    *	Override the toString() method of java.lang.Object for printing Elevator values.
	*
    *	@return		summary of transaction.
    */
	public String toString() {
        
        switch (fTransactionType) {
            case DEPOSIT:
                return "Deposit of $" + fAmount + " into account " + 
                        convertAcnToStr(fSourceAcc.getAccountNum());

            case WITHDRAWAL:
                return "Withdrawal of $" + fAmount + " from account " + 
                        convertAcnToStr(fSourceAcc.getAccountNum());

            case BALANCEINQUIRY:
                return "Balance Inquiry of account " + 
                        convertAcnToStr(fSourceAcc.getAccountNum());

            case TRANSFER:
                return "Transfer of $"+ fAmount +" from account " + 
                        convertAcnToStr(fSourceAcc.getAccountNum())
                         + " to account " + 
                        convertAcnToStr(fTargetAcc.getAccountNum());
        }
        
		return ("Unknown transaction type");
        
	}
    
    
    /**																		   |
    *
    *	Convert an account number [int] to a string
    *	with preceeding "0"s.
    *
    *	@return		string version of account numbr with leading "0"s
	*/
    private String convertAcnToStr(int mAccountNum) {
    
        String  mReturn = "" + mAccountNum;
        int    mCounter = ACCOUNTSTRLENGTH - mReturn.length();
        
        while (mCounter > 0) {						// add preceeding "0"s
            mReturn = "0" + mReturn;
            mCounter--;
        }
        
        return mReturn;
    }
    
    
    /**																		   |
    * 
    *	Accessors.
    *
    */
    public int getTransactionType() {
        return fTransactionType;
    }
	public AccountInfo getSourceAcc() {
        return fSourceAcc;
    }
	public AccountInfo getTargetAcc() {
        return fTargetAcc;
    }
	public float getAmount() {
        return fAmount;
    }
	public Object getSource() {
        return fSource;
	}
}